@@ -2084,7 +2084,7 @@ g_socket_send_to (GSocket         *socket,
  *
  * Shut down part of a full-duplex connection.
  *
- * If @shutdown_read is %TRUE then the recieving side of the connection
+ * If @shutdown_read is %TRUE then the receiving side of the connection
  * is shut down, and further reading is disallowed.
  *
  * If @shutdown_write is %TRUE then the sending side of the connection
@@ -2439,7 +2439,6 @@ socket_source_prepare (GSource *source,
       if (*timeout < 0)
         {
           socket_source->socket->priv->timed_out = TRUE;
-          socket_source->pollfd.revents = socket_source->condition & (G_IO_IN | G_IO_OUT);
           *timeout = 0;
           return TRUE;
         }
@@ -2476,6 +2475,8 @@ socket_source_dispatch (GSource     *source,
 #ifdef G_OS_WIN32
   socket_source->pollfd.revents = update_condition (socket_source->socket);
 #endif
+  if (socket_source->socket->priv->timed_out)
+    socket_source->pollfd.revents |= socket_source->condition & (G_IO_IN | G_IO_OUT);
 
   return (*func) (socket_source->socket,
 		  socket_source->pollfd.revents & socket_source->condition,
