@@ -54,11 +54,16 @@
 #include <unistd.h>
 #endif
 
+#ifdef HAVE_LANGINFO_TIME
+#include <langinfo.h>
+#endif
+
 #include "gdatetime.h"
 
 #include "gatomic.h"
 #include "gfileutils.h"
 #include "ghash.h"
+#include "giochannel.h"
 #include "gmain.h"
 #include "gmappedfile.h"
 #include "gstrfuncs.h"
@@ -71,6 +76,9 @@
 #ifndef G_OS_WIN32
 #include <sys/time.h>
 #include <time.h>
+#ifdef HAVE_TIMERFD
+#include <sys/timerfd.h>
+#endif
 #endif /* !G_OS_WIN32 */
 
 /**
@@ -140,38 +148,16 @@ struct _GDateTime
 #define USEC_PER_DAY         (G_GINT64_CONSTANT (86400000000))
 #define SEC_PER_DAY          (G_GINT64_CONSTANT (86400))
 
-#define GREGORIAN_LEAP(y)    ((((y) % 4) == 0) && (!((((y) % 100) == 0) && (((y) % 400) != 0))))
-#define JULIAN_YEAR(d)       ((d)->julian / 365.25)
-#define DAYS_PER_PERIOD      (G_GINT64_CONSTANT (2914695))
-
-#define GET_AMPM(d,l)         ((g_date_time_get_hour (d) < 12)  \
-                                       /* Translators: 'before midday' indicator */ \
-                                ? (l ? C_("GDateTime", "am") \
-                                       /* Translators: 'before midday' indicator */ \
-                                     : C_("GDateTime", "AM")) \
-                                  /* Translators: 'after midday' indicator */ \
-                                : (l ? C_("GDateTime", "pm") \
-                                  /* Translators: 'after midday' indicator */ \
-                                     : C_("GDateTime", "PM")))
-
-#define WEEKDAY_ABBR(d)       (get_weekday_name_abbr (g_date_time_get_day_of_week (datetime)))
-#define WEEKDAY_FULL(d)       (get_weekday_name (g_date_time_get_day_of_week (datetime)))
-
-#define MONTH_ABBR(d)         (get_month_name_abbr (g_date_time_get_month (datetime)))
-#define MONTH_FULL(d)         (get_month_name (g_date_time_get_month (datetime)))
-
-/* Translators: this is the preferred format for expressing the date */
-#define GET_PREFERRED_DATE(d) (g_date_time_format ((d), C_("GDateTime", "%m/%d/%y")))
-
-/* Translators: this is the preferred format for expressing the time */
-#define GET_PREFERRED_TIME(d) (g_date_time_format ((d), C_("GDateTime", "%H:%M:%S")))
-
 #define SECS_PER_MINUTE (60)
 #define SECS_PER_HOUR   (60 * SECS_PER_MINUTE)
 #define SECS_PER_DAY    (24 * SECS_PER_HOUR)
 #define SECS_PER_YEAR   (365 * SECS_PER_DAY)
 #define SECS_PER_JULIAN (DAYS_PER_PERIOD * SECS_PER_DAY)
 
+#define GREGORIAN_LEAP(y)    ((((y) % 4) == 0) && (!((((y) % 100) == 0) && (((y) % 400) != 0))))
+#define JULIAN_YEAR(d)       ((d)->julian / 365.25)
+#define DAYS_PER_PERIOD      (G_GINT64_CONSTANT (2914695))
+
 static const guint16 days_in_months[2][13] =
 {
   { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 },
@@ -184,6 +170,52 @@ static const guint16 days_in_year[2][13] =
   {  0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366 }
 };
 
+#ifdef HAVE_LANGINFO_TIME
+
+#define GET_AMPM(d) ((g_date_time_get_hour (d) < 12) ? \
+                     nl_langinfo (AM_STR) : \
+                     nl_langinfo (PM_STR))
+
+#define PREFERRED_DATE_FMT nl_langinfo (D_FMT)
+#define PREFERRED_TIME_FMT nl_langinfo (T_FMT)
+
+static const gint weekday_item[2][7] =
+{
+  { ABDAY_2, ABDAY_3, ABDAY_4, ABDAY_5, ABDAY_6, ABDAY_7, ABDAY_1 },
+  { DAY_2, DAY_3, DAY_4, DAY_5, DAY_6, DAY_7, DAY_1 }
+};
+
+static const gint month_item[2][12] =
+{
+  { ABMON_1, ABMON_2, ABMON_3, ABMON_4, ABMON_5, ABMON_6, ABMON_7, ABMON_8, ABMON_9, ABMON_10, ABMON_11, ABMON_12 },
+  { MON_1, MON_2, MON_3, MON_4, MON_5, MON_6, MON_7, MON_8, MON_9, MON_10, MON_11, MON_12 },
+};
+
+#define WEEKDAY_ABBR(d) nl_langinfo (weekday_item[0][g_date_time_get_day_of_week (d) - 1])
+#define WEEKDAY_FULL(d) nl_langinfo (weekday_item[1][g_date_time_get_day_of_week (d) - 1])
+#define MONTH_ABBR(d) nl_langinfo (month_item[0][g_date_time_get_month (d) - 1])
+#define MONTH_FULL(d) nl_langinfo (month_item[1][g_date_time_get_month (d) - 1])
+
+#else
+
+#define GET_AMPM(d)          ((g_date_time_get_hour (d) < 12)  \
+                                       /* Translators: 'before midday' indicator */ \
+                                ? C_("GDateTime", "am") \
+                                  /* Translators: 'after midday' indicator */ \
+                                : C_("GDateTime", "pm"))
+
+/* Translators: this is the preferred format for expressing the date */
+#define PREFERRED_DATE_FMT C_("GDateTime", "%m/%d/%y")
+
+/* Translators: this is the preferred format for expressing the time */
+#define PREFERRED_TIME_FMT C_("GDateTime", "%H:%M:%S")
+
+
+#define WEEKDAY_ABBR(d)       (get_weekday_name_abbr (g_date_time_get_day_of_week (d)))
+#define WEEKDAY_FULL(d)       (get_weekday_name (g_date_time_get_day_of_week (d)))
+#define MONTH_ABBR(d)         (get_month_name_abbr (g_date_time_get_month (d)))
+#define MONTH_FULL(d)         (get_month_name (g_date_time_get_month (d)))
+
 static const gchar *
 get_month_name (gint month)
 {
@@ -312,6 +344,8 @@ get_weekday_name_abbr (gint day)
   return NULL;
 }
 
+#endif  /* HAVE_LANGINFO_TIME */
+
 static inline gint
 ymd_to_days (gint year,
              gint month,
@@ -1601,15 +1635,15 @@ g_date_time_get_day_of_month (GDateTime *datetime)
  * within a complete week (Monday to Sunday) is contained within the
  * same week-numbering year.
  *
- * For Monday, Tuesday and Wednesday occuring near the end of the year,
+ * For Monday, Tuesday and Wednesday occurring near the end of the year,
  * this may mean that the week-numbering year is one greater than the
  * calendar year (so that these days have the same week-numbering year
- * as the Thursday occuring early in the next year).
+ * as the Thursday occurring early in the next year).
  *
- * For Friday, Saturaday and Sunday occuring near the start of the year,
+ * For Friday, Saturaday and Sunday occurring near the start of the year,
  * this may mean that the week-numbering year is one less than the
  * calendar year (so that these days have the same week-numbering year
- * as the Thursday occuring late in the previous year).
+ * as the Thursday occurring late in the previous year).
  *
  * An equivalent description is that the week-numbering year is equal to
  * the calendar year containing the majority of the days in the current
@@ -1672,7 +1706,7 @@ g_date_time_get_week_numbering_year (GDateTime *datetime)
  * that has more than 4 of its days falling within the calendar year.
  *
  * The value 0 is never returned by this function.  Days contained
- * within a year but occuring before the first ISO 8601 week of that
+ * within a year but occurring before the first ISO 8601 week of that
  * year are considered as being contained in the last week of the
  * previous year.  Similarly, the final days of a calendar year may be
  * considered as being part of the first ISO 8601 week of the next year
@@ -2222,6 +2256,11 @@ get_numeric_format (gchar    *fmt,
  *    a tab character
  *  </simpara></listitem></varlistentry>
  *  <varlistentry><term>
+ *    <literal>%%T</literal>:
+ *   </term><listitem><simpara>
+ *    the time in 24-hour notation with seconds (<literal>%%H:%%M:%%S</literal>)
+ *  </simpara></listitem></varlistentry>
+ *  <varlistentry><term>
  *    <literal>%%u</literal>:
  *   </term><listitem><simpara>
  *    the day of the week as a decimal, range 1 to 7, Monday being 1
@@ -2312,8 +2351,8 @@ get_numeric_format (gchar    *fmt,
  * Since: 2.26
  */
 gchar *
-g_date_time_format (GDateTime *datetime,
-                    const gchar     *format)
+g_date_time_format (GDateTime   *datetime,
+                    const gchar *format)
 {
   GString  *outstr;
   gchar    *tmp;
@@ -2323,6 +2362,7 @@ g_date_time_format (GDateTime *datetime,
   gboolean  pad_set = FALSE;
   gchar     pad = '\0';
   gchar     fmt[20];
+  gchar    *ampm;
 
   g_return_val_if_fail (datetime != NULL, NULL);
   g_return_val_if_fail (format != NULL, NULL);
@@ -2425,21 +2465,27 @@ g_date_time_format (GDateTime *datetime,
                   alt_digits = TRUE;
                   goto next_mod;
                 case 'p':
-                  g_string_append (outstr, GET_AMPM (datetime, FALSE));
+                  ampm = g_utf8_strup (GET_AMPM (datetime), -1);
+                  g_string_append (outstr, ampm);
+                  g_free (ampm);
                   break;
                 case 'P':
-                  g_string_append (outstr, GET_AMPM (datetime, TRUE));
+                  ampm = g_utf8_strdown (GET_AMPM (datetime), -1);
+                  g_string_append (outstr, ampm);
+                  g_free (ampm);
                   break;
                 case 'r':
                   {
                     gint hour = g_date_time_get_hour (datetime) % 12;
                     if (hour == 0)
                       hour = 12;
+                    ampm = g_utf8_strup (GET_AMPM (datetime), -1);
                     g_string_append_printf (outstr, "%02d:%02d:%02d %s",
                                             hour,
                                             g_date_time_get_minute (datetime),
                                             g_date_time_get_second (datetime),
-                                            GET_AMPM (datetime, FALSE));
+                                            ampm);
+                    g_free (ampm);
                   }
                   break;
                 case 'R':
@@ -2457,6 +2503,12 @@ g_date_time_format (GDateTime *datetime,
                 case 't':
                   g_string_append_c (outstr, '\t');
                   break;
+                case 'T':
+                  g_string_append_printf (outstr, "%02d:%02d:%02d",
+                                          g_date_time_get_hour (datetime),
+                                          g_date_time_get_minute (datetime),
+                                          g_date_time_get_second (datetime));
+                  break;
                 case 'u':
                   get_numeric_format (fmt, sizeof(fmt), alt_digits, 0, 0);
                   g_string_append_printf (outstr, fmt, g_date_time_get_day_of_week (datetime));
@@ -2467,14 +2519,14 @@ g_date_time_format (GDateTime *datetime,
                   break;
                 case 'x':
                   {
-                    tmp = GET_PREFERRED_DATE (datetime);
+                    tmp = g_date_time_format (datetime, PREFERRED_DATE_FMT);
                     g_string_append (outstr, tmp);
                     g_free (tmp);
                   }
                   break;
                 case 'X':
                   {
-                    tmp = GET_PREFERRED_TIME (datetime);
+                    tmp = g_date_time_format (datetime, PREFERRED_TIME_FMT);
                     g_string_append (outstr, tmp);
                     g_free (tmp);
                   }
@@ -2539,6 +2591,234 @@ bad_format:
   return NULL;
 }
 
+typedef struct _GDateTimeSource GDateTimeSource;
+struct _GDateTimeSource
+{
+  GSource     source;
+  
+  gint64      real_expiration;
+  gint64      wakeup_expiration;
+
+  gboolean    cancel_on_set;
+
+  GPollFD     pollfd;
+};
+
+static inline void
+g_datetime_source_reschedule (GDateTimeSource *datetime_source,
+			     gint64                from_monotonic)
+{
+  datetime_source->wakeup_expiration = from_monotonic + G_TIME_SPAN_SECOND;
+}
+
+static gboolean
+g_datetime_source_is_expired (GDateTimeSource *datetime_source)
+{
+  gint64 real_now;
+
+  real_now = g_get_real_time ();
+  
+  if (datetime_source->real_expiration <= real_now)
+    return TRUE;
+
+  /* We can't really detect without system support when things change;
+   * so just trigger every second.
+   */
+  if (datetime_source->cancel_on_set)
+    return TRUE;
+
+  return FALSE;
+}
+
+/* In prepare, we're just checking the monotonic time against
+ * our projected wakeup.
+ */
+static gboolean
+g_datetime_source_prepare (GSource *source,
+			  gint    *timeout)
+{
+  GDateTimeSource *datetime_source = (GDateTimeSource*)source;
+  gint64 monotonic_now;
+
+#ifdef HAVE_TIMERFD
+  if (datetime_source->pollfd.fd != -1)
+    {
+      *timeout = -1;
+      return FALSE;
+    }
+#endif
+
+  monotonic_now = g_source_get_time (source);
+
+  if (monotonic_now < datetime_source->wakeup_expiration)
+    {
+      /* Round up to ensure that we don't try again too early */
+      *timeout = (datetime_source->wakeup_expiration - monotonic_now + 999) / 1000;
+      return FALSE;
+    }
+
+  *timeout = 0;
+  return g_datetime_source_is_expired (datetime_source);
+}
+
+/* In check, we're looking at the wall clock.
+ */
+static gboolean 
+g_datetime_source_check (GSource  *source)
+{
+  GDateTimeSource *datetime_source = (GDateTimeSource*)source;
+
+#ifdef HAVE_TIMERFD
+  if (datetime_source->pollfd.fd != -1)
+    return datetime_source->pollfd.revents != 0;
+#endif
+
+  if (g_datetime_source_is_expired (datetime_source))
+    return TRUE;
+
+  g_datetime_source_reschedule (datetime_source, g_source_get_time (source));
+  
+  return FALSE;
+}
+
+static gboolean
+g_datetime_source_dispatch (GSource    *source, 
+			   GSourceFunc callback,
+			   gpointer    user_data)
+{
+  if (!callback)
+    {
+      g_warning ("Timeout source dispatched without callback\n"
+                 "You must call g_source_set_callback().");
+      return FALSE;
+    }
+  
+  (callback) (user_data);
+
+  /* Always false as this source is documented to run once */
+  return FALSE;
+}
+
+static void
+g_datetime_source_finalize (GSource *source)
+{
+#ifdef HAVE_TIMERFD
+  GDateTimeSource *datetime_source = (GDateTimeSource*)source;
+  if (datetime_source->pollfd.fd != -1)
+    close (datetime_source->pollfd.fd);
+#endif
+}
+
+static GSourceFuncs g_datetime_source_funcs = {
+  g_datetime_source_prepare,
+  g_datetime_source_check,
+  g_datetime_source_dispatch,
+  g_datetime_source_finalize
+};
+
+#ifdef HAVE_TIMERFD
+static gboolean
+g_datetime_source_init_timerfd (GDateTimeSource *datetime_source,
+				gint64           unix_seconds)
+{
+  struct itimerspec its;
+  int settime_flags;
+
+  datetime_source->pollfd.fd = timerfd_create (CLOCK_REALTIME, TFD_CLOEXEC);
+  if (datetime_source->pollfd.fd == -1)
+    return FALSE;
+
+  memset (&its, 0, sizeof (its));
+  its.it_value.tv_sec = (time_t) unix_seconds;
+
+  /* http://article.gmane.org/gmane.linux.kernel/1132138 */
+#ifndef TFD_TIMER_CANCEL_ON_SET
+#define TFD_TIMER_CANCEL_ON_SET (1 << 1)
+#endif
+
+  settime_flags = TFD_TIMER_ABSTIME;
+  if (datetime_source->cancel_on_set)
+    settime_flags |= TFD_TIMER_CANCEL_ON_SET;
+
+  if (timerfd_settime (datetime_source->pollfd.fd, settime_flags, &its, NULL) < 0)
+    {
+      close (datetime_source->pollfd.fd);
+      datetime_source->pollfd.fd = -1;
+      return FALSE;
+    }
+
+  datetime_source->pollfd.events = G_IO_IN;
+
+  g_source_add_poll ((GSource*) datetime_source, &datetime_source->pollfd);
+
+  return TRUE;
+}
+#endif
+
+/**
+ * g_date_time_source_new:
+ * @datetime: Time to await
+ * @cancel_on_set: Also invoke callback if the system clock changes discontiguously
+ *
+ * This function is designed for programs that want to schedule an
+ * event based on real (wall clock) time, as returned by
+ * g_get_real_time().  For example, HOUR:MINUTE wall-clock displays
+ * and calendaring software.  The callback will be invoked when the
+ * specified wall clock time @datetime is reached.  This includes
+ * events such as the system clock being set past the given time.
+ *
+ * Compare versus g_timeout_source_new() which is defined to use
+ * monotonic time as returned by g_get_monotonic_time().
+ *
+ * If @cancel_on_set is given, the callback will also be invoked at
+ * most a second after the system clock is changed.  This includes
+ * being set backwards or forwards, and system
+ * resume from suspend.  Not all operating systems allow detecting all
+ * relevant events efficiently - this function may cause the process
+ * to wake up once a second in those cases.
+ *
+ * A wall clock display should use @cancel_on_set; a calendaring
+ * program shouldn't need to.
+ *
+ * Note that the return value from the associated callback will be
+ * ignored; this is a one time watch.
+ *
+ * <note><para>This function currently does not detect time zone
+ * changes.  On Linux, your program should also monitor the
+ * <literal>/etc/timezone</literal> file using
+ * #GFileMonitor.</para></note>
+*
+ * <example id="gdatetime-example-watch"><title>Clock example</title><programlisting><xi:include xmlns:xi="http://www.w3.org/2001/XInclude" parse="text" href="../../../../glib/tests/glib-clock.c"><xi:fallback>FIXME: MISSING XINCLUDE CONTENT</xi:fallback></xi:include></programlisting></example>
+ *
+ * Return value: A newly-constructed #GSource
+ *
+ * Since: 2.30
+ **/
+GSource *
+g_date_time_source_new (GDateTime  *datetime,
+			gboolean    cancel_on_set)
+{
+  GDateTimeSource *datetime_source;
+  gint64 unix_seconds;
+
+  unix_seconds = g_date_time_to_unix (datetime);
+
+  datetime_source = (GDateTimeSource*) g_source_new (&g_datetime_source_funcs, sizeof (GDateTimeSource));
+
+  datetime_source->cancel_on_set = cancel_on_set;
+
+#ifdef HAVE_TIMERFD
+  if (g_datetime_source_init_timerfd (datetime_source, unix_seconds))
+    return (GSource*)datetime_source;
+  /* Fall through to non-timerfd code */
+#endif
+
+  datetime_source->real_expiration = unix_seconds * 1000000;
+  g_datetime_source_reschedule (datetime_source, g_get_monotonic_time ());
+
+  return (GSource*)datetime_source;
+}
+
 
 /* Epilogue {{{1 */
 /* vim:set foldmethod=marker: */
